@@ -24,7 +24,6 @@
 #'    *Ann. Appl. Stat.* **13** (2019), no. 4, 2065--2090 \doi{10.1214/19-AOAS1269}
 #'
 #'
-#' @importFrom lifecycle deprecate_stop deprecate_stop deprecated is_present
 #' @importFrom rlang abort
 enpy_initial_estimates <- function (x, y, alpha, lambda, bdp = 0.25, cc, intercept = TRUE, penalty_loadings,
                                     enpy_opts = enpy_options(), mscale_opts = mscale_algorithm_options(),
@@ -95,8 +94,6 @@ enpy_initial_estimates <- function (x, y, alpha, lambda, bdp = 0.25, cc, interce
 #' @param y vector of response values of length `n`.
 #' @inheritParams pense
 #' @param en_algorithm_opts options for the LS-EN algorithm. See [en_algorithm_options] for details.
-#' @param method defunct. PSCs are always computed for EN estimates. For the PY procedure for unpenalized estimation
-#'    use package [pyinit](https://cran.r-project.org/package=pyinit).
 #'
 #' @return a list of principal sensitivity components, one per element in `lambda`. Each PSC is itself a list
 #'    with items `lambda`, `alpha`, and `pscs`.
@@ -112,10 +109,9 @@ enpy_initial_estimates <- function (x, y, alpha, lambda, bdp = 0.25, cc, interce
 #'    A Fast Procedure for Outlier Diagnostics in Large Regression Problems.
 #'    *J. Amer. Statist. Assoc.* **94** (1999). no. 446, 434--445. \doi{10.2307/2670164}
 #'
-#' @importFrom lifecycle deprecate_stop deprecate_stop deprecated is_present
 #' @importFrom rlang abort
 prinsens <- function (x, y, alpha, lambda, intercept = TRUE, penalty_loadings, en_algorithm_opts,
-                      eps = 1e-6, sparse = FALSE, ncores = 1L, method = deprecated()) {
+                      eps = 1e-6, sparse = FALSE, ncores = 1L) {
   y <- .as(y, 'numeric')
   x_dim <- dim(x)
 
@@ -125,14 +121,6 @@ prinsens <- function (x, y, alpha, lambda, intercept = TRUE, penalty_loadings, e
     abort("`x` must be a matrix with at least 1 column.")
   }
 
-  if (is_present(method)) {
-    if (method == 'en') {
-      deprecate_stop('2.0.0', 'prinsens(method = )')
-    } else {
-      deprecate_stop('2.0.0', 'prinsens(method = )', details = 'For unpenalized estimates use the pyinit package.')
-    }
-  }
-
   sparse <- .as(sparse[[1L]], 'logical')
 
   if (missing(en_algorithm_opts)) {
