@@ -15,11 +15,13 @@
 plot.pense_fit <- function (x, alpha, ...) {
   if (missing(alpha) || is.null(alpha)) {
     alpha <- x$alpha[[1L]]
-  }
-  alpha <- .as(alpha[[1L]], 'numeric')
-  ai <- which((x$alpha - alpha)^2 < .Machine$double.eps)
-  if (length(ai) != 1L) {
-    abort("Requested `alpha` is not available in the fit.")
+    ai <- 1L
+  } else {
+    alpha <- .as(alpha[[1L]], 'numeric')
+    ai <- which((x$alpha - alpha)^2 < .Machine$double.eps)
+    if (length(ai) != 1L) {
+      abort("Requested `alpha` is not available in the fit.")
+    }
   }
   .plot_coef_path(x, lambda_seq = x$lambda[[ai]], alpha = alpha, envir = parent.frame())
   invisible(x)
@@ -84,7 +86,9 @@ plot.pense_cvfit <- function(x, what = c('cv', 'coef.path'), alpha = NULL, se_mu
 #' @importFrom graphics plot segments abline
 #' @importFrom rlang abort warn
 .plot_cv_res <- function (object, alpha = NULL, se_mult) {
-  measure_label <- switch(object$cv_measure, mape = "Median absolute prediction error",
+  measure_label <- switch(object$cv_measure,
+                          RIS = "Weighted mean square prediction error",
+                          mape = "Median absolute prediction error",
                           rmspe = "Root mean square prediction error",
                           auroc = "1 - AUROC",
                           tau_size = expression(paste(tau, "-scale of the prediction error")),
@@ -172,22 +176,22 @@ plot.pense_cvfit <- function(x, what = c('cv', 'coef.path'), alpha = NULL, se_mu
   }
   alpha <- .as(alpha[[1L]], 'numeric')
 
-  var_names <- tryCatch(colnames(eval(object$call$x, envir = envir)), error = function (e) NULL)
+  var_names <- tryCatch(colnames(eval(object$call$x, envir = envir)), error = \(e) NULL)
   if (is.null(var_names)) {
-    var_names <- paste('X', seq_len(length(object$estimates[[1]]$beta)), sep = '')
+    var_names <- paste('X', seq_len(length(object$estimates[[1]][[1]]$beta)), sep = '')
   }
 
-  match_ests <- which(vapply(object$estimates, FUN.VALUE = logical(1L), FUN = function (est) {
-    (est$alpha - alpha)^2 < .Machine$double.eps
+  match_ests <- which(vapply(object$estimates, FUN.VALUE = logical(1L), FUN = \(est) {
+    (est[[1L]]$alpha - alpha)^2 < .Machine$double.eps
   }))
   if (length(match_ests) == 0L) {
     abort("Requested `alpha` not available in the fit.")
   }
 
-  active_vars <- do.call(rbind, lapply(object$estimates[match_ests], function (est) {
-    actives <- .active_indices_and_values(est)
+  active_vars <- do.call(rbind, lapply(object$estimates[match_ests], \(est) {
+    actives <- .active_indices_and_values(est[[1L]])
     if (length(actives$var_index) > 0L) {
-      return(data.frame(actives, lambda = est$lambda, alpha = est$alpha))
+      return(data.frame(actives, lambda = est[[1L]]$lambda, alpha = est[[1L]]$alpha))
     } else {
       return(data.frame(var_index = integer(), value = numeric(),
                         lambda = numeric(), alpha = numeric()))
