@@ -3,15 +3,16 @@
 //  pense
 //
 //  Created by David Kepplinger on 2019-01-30.
-//  Copyright © 2019 David Kepplinger. All rights reserved.
+//  Copyright © 2016 David Kepplinger. All rights reserved.
 //
 
-#ifndef REGULARIZATION_PATH_NEW_HPP_
-#define REGULARIZATION_PATH_NEW_HPP_
+#ifndef REGULARIZATION_PATH_HPP_
+#define REGULARIZATION_PATH_HPP_
 
 #include <memory>
 #include <tuple>
 #include <type_traits>
+#include <cstdint>
 
 #include "nsoptim.hpp"
 
@@ -45,10 +46,20 @@ bool CoefficientsEquivalent(const Coefficients& a, const Coefficients& b,
   return false;
 }
 
+enum class InsertResult { kGood, kBad, kDuplicate };
+
+enum class TupleComparison : std::int8_t {
+  kLowerObjf = -2,
+    kSlightlyLowerObjf = -1,
+    kEqualObjf = 0,
+    kSlightlyHigherObjf = 1,
+    kHigherObjf = 2
+};
+
 //! A list of starting points with associated optimizer.
 template<class Ordering, typename... Ts>
 class OrderedTuples {
- public:
+public:
   using Element = std::tuple<Ts...>;
 
   enum class InsertResult { kGood, kBad, kDuplicate };
@@ -82,8 +93,8 @@ class OrderedTuples {
 
   //! Move constructor.
   OrderedTuples(OrderedTuples&& other) noexcept :
-      max_size_(other.max_size_), order_(std::move(other.order_)),
-      size_(other.size_), elements_(std::move(other.elements_)) {
+    max_size_(other.max_size_), order_(std::move(other.order_)),
+    size_(other.size_), elements_(std::move(other.elements_)) {
     other.size_ = 0;
   }
   OrderedTuples& operator=(OrderedTuples&&) = delete;
@@ -103,32 +114,53 @@ class OrderedTuples {
   // template<typename... Args>
   InsertResult Emplace(Ts&&... args) {
     // Check if the optimum's objective function value is good enough.
-    if (max_size_ > 0 && size_ >= max_size_ &&
-          order_.after(elements_.front(), std::forward<Ts>(args)...)) {
-      return InsertResult::kBad;
+    if (max_size_ > 0 && size_ >= max_size_) {
+      auto first_compared_to_new = order_.CompareObjf(elements_.front(), std::forward<Ts>(args)...);
+      if (first_compared_to_new < TupleComparison::kEqualObjf) {
+        return InsertResult::kBad;
+      }
     }
 
     // Determine insert position.
     const auto elements_end = elements_.end();
     auto current_it = elements_.begin();
     auto insert_it = elements_.before_begin();
+    // auto before_insert_it = elements_.before_begin();
 
     while (current_it != elements_end) {
-      const bool better = order_.before(*current_it, std::forward<Ts>(args)...);
+      auto objf_comparison = order_.CompareObjf(*current_it, std::forward<Ts>(args)...);
+      // const bool better = order_.before(*current_it, std::forward<Ts>(args)...);
 
-      // Check if the coefficients are equal to the element at question.
-      if (!better && !order_.after(*current_it, std::forward<Ts>(args)...) &&
-          order_.equivalent(*current_it, std::forward<Ts>(args)...)) {
-        return InsertResult::kDuplicate;
-      } else if (!better) {
+      if (objf_comparison < TupleComparison::kSlightlyLowerObjf) {
+        // current element has much better objective function value. Insert here.
         break;
+      } else if (objf_comparison <= TupleComparison::kSlightlyHigherObjf) {
+        // current element has similar objective function as the new element
+        const bool equivalent = order_.Equivalent(*current_it, std::forward<Ts>(args)...);
+        if (equivalent) {
+          if (objf_comparison > TupleComparison::kEqualObjf) {
+            // current element is equivalent, but the the new element is slightly better.
+            // Replace.
+            // *current_it = Element(std::forward<Ts>(args)...);
+            auto updated_it = elements_.emplace_after(insert_it, std::forward<Ts>(args)...);
+            elements_.erase_after(updated_it);
+            return InsertResult::kGood;
+          } else {
+            return InsertResult::kDuplicate;
+          }
+        } else if (objf_comparison < TupleComparison::kEqualObjf) {
+          // current element is not equivalent and has slightly better objective function.
+          // Insert here.
+          break;
+        }
+        // else current element is not equivalent and slightly worse than the new one.
       }
+      // else current element is worse. Continue search.
 
       ++current_it;
       ++insert_it;
     }
 
-    // No duplicate has been detected. Add after the insert position.
     elements_.emplace_after(insert_it, std::forward<Ts>(args)...);
     // Ensure that the size stays within the limits.
     if (++size_ > max_size_ && max_size_ > 0) {
@@ -146,7 +178,7 @@ class OrderedTuples {
     return elements_;
   }
 
- private:
+private:
   const size_t max_size_;
   Ordering order_;
   size_t size_;
@@ -155,28 +187,22 @@ class OrderedTuples {
 
 template<class Coefficients>
 class DuplicateCoefficients {
- public:
+public:
   explicit DuplicateCoefficients(const double eps) noexcept : eps_(eps) {}
 
-  //! Does the existing element come before the new element?
+  //! Always returns `kLowerObjf` since coefficients do not have an associated objective function value
   template<typename Element, typename... Args>
-  bool before(const Element&, const Coefficients&, Args&&...) const noexcept {
-    return false;
-  }
-
-  //! Does the existing element come before the new element?
-  template<typename Element, typename... Args>
-  bool after(const Element&, const Coefficients&, Args&&...) const noexcept {
-    return false;
+  TupleComparison CompareObjf(const Element& el, const Coefficients& coefs, Args&&...) const noexcept {
+    return TupleComparison::kSlightlyLowerObjf;
   }
 
   //! Is the existing element equivalent to new element?
   template<typename Element, typename... Args>
-  bool equivalent(const Element& el, const Coefficients& coefs, Args&&...) const noexcept {
+  bool Equivalent(const Element& el, const Coefficients& coefs, Args&&...) const noexcept {
     return CoefficientsEquivalent(std::get<0>(el), coefs, eps_);
   }
 
- private:
+private:
   const double eps_;
 };
 
@@ -184,48 +210,64 @@ template<class Optimizer>
 class OptimaOrder {
   using Coefficients = typename Optimizer::Coefficients;
   using Optimum = typename Optimizer::Optimum;
- public:
+public:
   explicit OptimaOrder(const double eps) noexcept : eps_(eps) {}
 
-  //! Does the existing element come before the new element?
+  //! Compare the value of the objective function of `el` against `opt.objf_value` in the form
+  //! `el.objf_value < opt.objf_value`.
+  //! Returns `kLowerObjf` if `el` has a lower value of the objective function than
+  //! `opt.objf_value` and `kHigherObjf` if `el` has a higher value of the objective function
+  //! than `opt.objf_value`.
   template<typename Element, typename... Args>
-  bool before(const Element& el, const Coefficients& coefs, const double objf_value,
-              Args&&...) const noexcept {
-    return std::get<1>(el) > objf_value + eps_;
+  TupleComparison CompareObjf(const Element& el, const Optimum& opt, Args&&...) const noexcept {
+    const double objf_value = std::get<0>(el).objf_value;
+    if (objf_value < opt.objf_value * (1 - eps_)) {
+      return TupleComparison::kLowerObjf;
+    } else if (objf_value < opt.objf_value) {
+      return TupleComparison::kSlightlyLowerObjf;
+    } else if (objf_value > opt.objf_value * (1 + eps_)) {
+      return TupleComparison::kHigherObjf;
+    } else if (objf_value > opt.objf_value) {
+      return TupleComparison::kSlightlyHigherObjf;
+    }
+    return TupleComparison::kEqualObjf;
   }
 
-  //! Does the existing element come before the new element?
+  //! Compare the value of the objective function of `el` against `objf_value` in the form
+  //! `el.objf_value < objf_value`.
+  //! Returns `kLowerObjf` if `el` has a lower value of the objective function than `objf_value` and
+  //! `kHigherObjf` if `el` has a higher value of the objective function than `objf_value`.
   template<typename Element, typename... Args>
-  bool before(const Element& el, const Optimum& opt, Args&&...) const noexcept {
-    return std::get<0>(el).objf_value > opt.objf_value + eps_;
-  }
-
-  //! Does the existing element come before the new element?
-  template<typename Element, typename... Args>
-  bool after(const Element& el, const Coefficients& coefs, const double objf_value,
-             Args&&...) const noexcept {
-    return std::get<1>(el) < objf_value - eps_;
-  }
-
-  //! Does the existing element come before the new element?
-  template<typename Element, typename... Args>
-  bool after(const Element& el, const Optimum& opt, Args&&...) const noexcept {
-    return std::get<0>(el).objf_value < opt.objf_value - eps_;
+  TupleComparison CompareObjf(const Element& el,
+                              const Coefficients& coefs,
+                              const double objf_value,
+                              Args&&...) const noexcept {
+    const double el_objf_value = std::get<1>(el);
+    if (el_objf_value < objf_value * (1 - eps_)) {
+      return TupleComparison::kLowerObjf;
+    } else if (el_objf_value < objf_value) {
+      return TupleComparison::kSlightlyLowerObjf;
+    } else if (el_objf_value > objf_value * (1 + eps_)) {
+      return TupleComparison::kHigherObjf;
+    } else if (el_objf_value > objf_value) {
+      return TupleComparison::kSlightlyHigherObjf;
+    }
+    return TupleComparison::kEqualObjf;
   }
 
   //! Is the existing element equivalent to new element?
   template<typename Element, typename... Args>
-  bool equivalent(const Element& el, const Coefficients& coefs, Args&&...) const noexcept {
+  bool Equivalent(const Element& el, const Coefficients& coefs, Args&&...) const noexcept {
     return CoefficientsEquivalent(std::get<0>(el), coefs, eps_);
   }
 
   //! Is the existing element equivalent to new element?
   template<typename Element, typename... Args>
-  bool equivalent(const Element& el, const Optimum& opt, Args&&...) const noexcept {
+  bool Equivalent(const Element& el, const Optimum& opt, Args&&...) const noexcept {
     return CoefficientsEquivalent(std::get<0>(el).coefs, opt.coefs, eps_);
   }
 
- private:
+private:
   const double eps_;
 };
 
@@ -259,7 +301,7 @@ class RegularizationPath {
   using BestOptima = regpath::UniqueOptima<Optimizer>;
   using BestOptimaOrder = regpath::OptimaOrder<Optimizer>;
 
- public:
+public:
   struct Solutions {
     const PenaltyFunction& penalty;
     alias::Optima<Optimizer> optima;
@@ -275,20 +317,20 @@ class RegularizationPath {
   RegularizationPath(const Optimizer& optimizer,
                      const PenaltyList& penalties, const int max_optima,
                      const double comparison_tol, const int num_threads) :
-      optimizer_template_(optimizer), penalties_(penalties),
-      max_optima_(max_optima), comparison_tol_(comparison_tol), num_threads_(num_threads),
-      shared_starts_(UniqueCoefficientsOrder(comparison_tol_)),
-      best_starts_(max_optima, BestOptimaOrder(comparison_tol)),
-      penalties_it_(penalties_.begin()) {
-        auto penalties_it = penalties_.before_begin();
-        const auto penalties_end = penalties_.end();
+    optimizer_template_(optimizer), penalties_(penalties),
+    max_optima_(max_optima), comparison_tol_(comparison_tol), num_threads_(num_threads),
+    shared_starts_(UniqueCoefficientsOrder(comparison_tol_)),
+    best_starts_(max_optima, BestOptimaOrder(comparison_tol)),
+    penalties_it_(penalties_.begin()) {
+    auto penalties_it = penalties_.before_begin();
+    const auto penalties_end = penalties_.end();
 
-        while (++penalties_it != penalties_end) {
-          individual_starts_.emplace_front(
-            UniqueCoefficients(UniqueCoefficientsOrder(comparison_tol_)));
-        }
-        individual_starts_it_ = individual_starts_.before_begin();
-      }
+    while (++penalties_it != penalties_end) {
+      individual_starts_.emplace_front(
+        UniqueCoefficients(UniqueCoefficientsOrder(comparison_tol_)));
+    }
+    individual_starts_it_ = individual_starts_.before_begin();
+  }
 
   //! Set the exploration options.
   //!
@@ -344,7 +386,7 @@ class RegularizationPath {
     return penalties_it_ == penalties_.end();
   }
 
- private:
+private:
   Optimizer optimizer_template_;
   const PenaltyList& penalties_;
   const int max_optima_;
@@ -381,75 +423,75 @@ class RegularizationPath {
     const auto is_end = individual_starts_it_->Elements().end();
     const auto sh_end = shared_starts_.Elements().end();
 
-    #pragma omp parallel \
-                num_threads(num_threads_) \
-                default(shared)
-    {
-      #pragma omp single nowait
-      for (auto is_it = individual_starts_it_->Elements().begin(); is_it != is_end; ++is_it) {
-        #pragma omp task \
-                    default(none) \
-                    firstprivate(is_it) \
-                    shared(explore_tol_, explore_it_) \
-                    shared(explored_solutions, optimizer_template_) const_local_shared(orig_tol)
-        {
-          Optimizer optimizer(optimizer_template_);
-          optimizer.convergence_tolerance(explore_tol_);
-          auto optimum = optimizer.Optimize(std::get<0>(*is_it), explore_it_);
-          optimizer.convergence_tolerance(orig_tol);
-
-          #pragma omp critical(insert_explored)
-          explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
-                                     std::move(optimizer), std::move(optimum.metrics));
-
-        }
-      }
-
-      #pragma omp single nowait
-      for (auto sh_it = shared_starts_.Elements().begin(); sh_it != sh_end; ++sh_it) {
-        #pragma omp task \
-                    firstprivate(sh_it) \
-                    default(none) \
-                    shared(explore_tol_, explore_it_) \
-                    shared(explored_solutions, optimizer_template_) const_local_shared(orig_tol)
-        {
-          Optimizer optimizer(optimizer_template_);
-          optimizer.convergence_tolerance(explore_tol_);
-          auto optimum = optimizer.Optimize(std::get<0>(*sh_it), explore_it_);
-          optimizer.convergence_tolerance(orig_tol);
-
-          #pragma omp critical(insert_explored)
-          explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
-                                     std::move(optimizer), std::move(optimum.metrics));
-
-        }
-      }
-
-      #pragma omp single nowait
-      if (use_warm_start_ || explored_solutions.Size() == 0) {
-      const auto bs_end = best_starts_.Elements().end();
-
-        for (auto bs_it = best_starts_.Elements().begin(); bs_it != bs_end; ++bs_it) {
-          #pragma omp task \
-                      firstprivate(bs_it) \
-                      default(none) \
-                      shared(explore_tol_, explore_it_, explored_solutions) \
-                      shared(optimizer_template_) const_local_shared(orig_tol, bs_end)
+#pragma omp parallel          \
+    num_threads(num_threads_) \
+      default(shared)
+      {
+#pragma omp single nowait
+        for (auto is_it = individual_starts_it_->Elements().begin(); is_it != is_end; ++is_it) {
+#pragma omp task                            \
+          default(none)                     \
+          firstprivate(is_it)               \
+          shared(explore_tol_, explore_it_) \
+          shared(explored_solutions, optimizer_template_) const_local_shared(orig_tol)
           {
-            auto&& optimizer = std::get<1>(*bs_it);
+            Optimizer optimizer(optimizer_template_);
             optimizer.convergence_tolerance(explore_tol_);
-            optimizer.penalty(optimizer_template_.penalty());
-            auto optimum = optimizer.Optimize(explore_it_);
+            auto optimum = optimizer.Optimize(std::get<0>(*is_it), explore_it_);
             optimizer.convergence_tolerance(orig_tol);
 
-            #pragma omp critical(insert_explored)
+#pragma omp critical(insert_explored)
             explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
                                        std::move(optimizer), std::move(optimum.metrics));
 
           }
         }
+
+#pragma omp single nowait
+        for (auto sh_it = shared_starts_.Elements().begin(); sh_it != sh_end; ++sh_it) {
+#pragma omp task                            \
+          firstprivate(sh_it)               \
+          default(none)                     \
+          shared(explore_tol_, explore_it_) \
+          shared(explored_solutions, optimizer_template_) const_local_shared(orig_tol)
+          {
+            Optimizer optimizer(optimizer_template_);
+            optimizer.convergence_tolerance(explore_tol_);
+            auto optimum = optimizer.Optimize(std::get<0>(*sh_it), explore_it_);
+            optimizer.convergence_tolerance(orig_tol);
+
+#pragma omp critical(insert_explored)
+            explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
+                                       std::move(optimizer), std::move(optimum.metrics));
+
+          }
+        }
+
+#pragma omp single nowait
+        if (use_warm_start_ || explored_solutions.Size() == 0) {
+          const auto bs_end = best_starts_.Elements().end();
+
+          for (auto bs_it = best_starts_.Elements().begin(); bs_it != bs_end; ++bs_it) {
+#pragma omp task                                                  \
+            firstprivate(bs_it)                                   \
+            default(none)                                         \
+            shared(explore_tol_, explore_it_, explored_solutions) \
+            shared(optimizer_template_) const_local_shared(orig_tol, bs_end)
+            {
+              auto&& optimizer = std::get<1>(*bs_it);
+              optimizer.convergence_tolerance(explore_tol_);
+              optimizer.penalty(optimizer_template_.penalty());
+              auto optimum = optimizer.Optimize(explore_it_);
+              optimizer.convergence_tolerance(orig_tol);
+
+#pragma omp critical(insert_explored)
+              explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
+                                         std::move(optimizer), std::move(optimum.metrics));
+
+            }
+          }
+        }
       }
-    }
 
     Rcpp::checkUserInterrupt();
     return explored_solutions;
@@ -489,7 +531,7 @@ class RegularizationPath {
         auto optimum = optimizer.Optimize(explore_it_);
         optimizer.convergence_tolerance(orig_tol);
         explored_solutions.Emplace(std::move(optimum.coefs), std::move(optimum.objf_value),
-                                  std::move(optimizer), std::move(optimum.metrics));
+                                   std::move(optimizer), std::move(optimum.metrics));
 
         Rcpp::checkUserInterrupt();
       }
@@ -518,7 +560,7 @@ class RegularizationPath {
         auto&& optimizer = std::get<1>(start);
         optimizer.penalty(optimizer_template_.penalty());
         explored_solutions.Emplace(std::move(std::get<0>(start).coefs), -1, std::move(optimizer),
-                                  MetricsPtr());
+                                   MetricsPtr());
       }
     }
     return explored_solutions;
@@ -544,7 +586,7 @@ class RegularizationPath {
     for (auto&& start : explored.Elements()) {
       auto&& optimizer = std::get<2>(start);
       auto optim = (std::get<1>(start) > 0) ?
-        optimizer.Optimize() :
+      optimizer.Optimize() :
         optimizer.Optimize(std::get<0>(start));
 
       if (optim.metrics && std::get<3>(start)) {
@@ -561,35 +603,35 @@ class RegularizationPath {
   void Concentrate(ExploredSolutions&& explored, std::true_type) {
     const auto ex_end = explored.Elements().end();
 
-    #pragma omp parallel \
-                num_threads(num_threads_) \
-                default(shared)
-    {
-      #pragma omp single nowait
-      for (auto ex_it = explored.Elements().begin(); ex_it != ex_end; ++ex_it) {
-        #pragma omp task \
-                    default(none) \
-                    firstprivate(ex_it) \
-                    shared(best_starts_)
-        {
-          auto&& optimizer = std::get<2>(*ex_it);
-          auto optim = (std::get<1>(*ex_it) > 0) ?
+#pragma omp parallel          \
+    num_threads(num_threads_) \
+      default(shared)
+      {
+#pragma omp single nowait
+        for (auto ex_it = explored.Elements().begin(); ex_it != ex_end; ++ex_it) {
+#pragma omp task              \
+          default(none)       \
+          firstprivate(ex_it) \
+          shared(best_starts_)
+          {
+            auto&& optimizer = std::get<2>(*ex_it);
+            auto optim = (std::get<1>(*ex_it) > 0) ?
             optimizer.Optimize() :
-            optimizer.Optimize(std::get<0>(*ex_it));
+              optimizer.Optimize(std::get<0>(*ex_it));
 
-          if (optim.metrics && std::get<3>(*ex_it)) {
-            auto&& exploration_metrics = optim.metrics->CreateSubMetrics("exploration");
-            exploration_metrics.AddSubMetrics(std::move(*std::get<3>(*ex_it)));
-            std::get<3>(*ex_it).reset();
+            if (optim.metrics && std::get<3>(*ex_it)) {
+              auto&& exploration_metrics = optim.metrics->CreateSubMetrics("exploration");
+              exploration_metrics.AddSubMetrics(std::move(*std::get<3>(*ex_it)));
+              std::get<3>(*ex_it).reset();
+            }
+#pragma omp critical(insert_concentrated)
+            best_starts_.Emplace(std::move(optim), std::move(optimizer));
           }
-          #pragma omp critical(insert_concentrated)
-          best_starts_.Emplace(std::move(optim), std::move(optimizer));
         }
       }
-    }
     Rcpp::checkUserInterrupt();
   }
 };
 } // namespace pense
 
-#endif // REGULARIZATION_PATH_NEW_HPP_
+#endif // REGULARIZATION_PATH_HPP_
